#!/bin/sh

####################################################################
#  Upgrades a 6100 Gen 4 from an old firmware version to a new one.
#  Copyright:    (c) 2014 Force America, Inc.
####################################################################


# Global variables.
LOGFILE=/var/log/6100_software_update.log
MAIN_DIR=/tmp/upgrade
SNAPSHOT_UPGRADE_DIR=/tmp/upgrade/DataLogger/snapshot_upgrade
PREUPGRADE_VERSION="$(apt-cache show 6100-svn | grep Version: | cut -c 10-)"


####################################################################
# Name:          backup_snapshot_database
# Description:   Copies the snapshot_database to a temp folder
#                before the upgrade runs
# Created:       12/2/2009 By Jon Polfer (P1131)
# Parameters:    None
# Returns:       Nothing
####################################################################
function backup_snapshot_database 
{
    # Copy snapshot.db to /tmp/upgrade/backup_snapshot.db
    cp /6100/etc/snapshot.db /tmp/upgrade/backup_snapshot.db
    cp /6100/etc/6100_Config.xml /tmp/upgrade/backup_6100_Config.xml
}

####################################################################
# Name:          restore_snapshot_database
# Description:   Copies the backed-up database to the 6100 folder. 
# Created:       12/2/2009 By Jon Polfer (P1131)
# Parameters:    None
# Returns:       Nothing
####################################################################
function restore_snapshot_database 
{
    # Copy /tmp/upgrade/backup_snapshot.db to /6100/etc/snapshot.db
    cp /tmp/upgrade/backup_snapshot.db /6100/etc/snapshot.db
    cp /tmp/upgrade/backup_6100_Config.xml /6100/etc/6100_Config.xml
    
}

####################################################################
# Name:          restore_upgrade_snapshot_database
# Description:   Restores the snapshot database to the 6100 tree,
#                and runs all upgrade scripts on it 
# Created:       12/2/2009 By Jon Polfer (P1131)
# Parameters:    None
# Returns:       Nothing
# Modified:      8/16/2010 By Jon Polfer (P1131)
#                - Revised script to upgrade to rev 8
####################################################################
function restore_upgrade_snapshot_database 
{
    # Copy /tmp/upgrade/backup_snapshot.db to /6100/etc/snapshot.db
    cp /tmp/upgrade/backup_snapshot.db /6100/etc/snapshot.db
    
 
    # Go back to the main directory
    cd $MAIN_DIR
}

####################################################################
#  Name:         cleanup_and_request_powerdown
#  Description:  Removes the /tmp/upgrade folder, cleans debian packages,
#                and informs the user of success or failure.
#                It then loops infinitely until the system is shut down.
#  Created:      By Andrew Dorney (P1131)
#  Parameters:   First parameter is a string, either "success" or "fail".
#                Parameter determines message sent to user.
####################################################################
function cleanup_and_request_powerdown {
	
	# Delete the /tmp/upgrade folder.
	# TODO - make the 6100 software do this.
	rm -rf /tmp/upgrade >> $LOGFILE 2>&1

	# If this function receives "success" as a parameter...
	if [[ "$1" = "success" ]]; then

		# Tell the logfile the upgrade was completed successfully.
		echo "Upgrade completed successfully!" >> $LOGFILE

		# Tell the user the upgrade was completed successfully.
		/6100/rc/pyclient set bool UpgradeSuccessful 1

		# Output a console message for old upgrades
		echo "Upgrade completed successfully!"
		echo "Shut off key power to the system."
	
	# If this function receives a "fail" as a parameter...
	elif [[ "$1" = "fail" ]]; then

		# Tell the logfile the upgrade was not completed successfully.
		echo "Upgrade was not completed successfully." >> $LOGFILE

		# Tell the user the upgrade was not completed successfully.
		/6100/rc/pyclient set bool UpgradeSuccessful ""

		# Output a console message for old upgrades
		echo "Upgrade failed!"
		echo "Shut off key power to the system."
	fi
	
	# Loop forever until the user shuts the system down.
	while true; do
		sleep 1
	done
}


####################################################################
# Name:          main (well, not really, but you get the point)
# Description:   Point of entry for the upgrade script.
# Parameters:    None.
# Created:       11/10/2009 By Andrew Dorney
####################################################################

# Move to MAIN_DIR to ensure we can find the debian packages.
cd $MAIN_DIR

# Check for and remove all archived Debian packages, to save space.
LIST_OF_ARCHIVED_DEBIAN_PACKAGES=$(ls /var/cache/apt/archives/*.deb)
if [[ "$LIST_OF_ARCHIVED_DEBIAN_PACKAGES" != "" ]]; then
    rm /var/cache/apt/archives/*.deb
fi


# Backup the snapshot (standard logging) database
backup_snapshot_database

# Stop the persistd daemon from running.  Attempt to stop database corruption
/etc/init.d/FA_persistd stop

# Install any debian packages that aren't on the system.
#
# DEBIAN_FRONTEND=noninteractive ensures that if a package tries to pop 
# up a menu (like samba-common does), we use its defaults and carry on.
#
# This doesn't validate pkg signatures (--no-debsig), forces a new file 
# to overwrite an older config file (--force-confnew), and doesn't 
# install downgrades, only newer packages (-G). -E saves us time and 
# hassle by not reinstalling packages we already have. See "man dpkg".
DEBIAN_FRONTEND=noninteractive dpkg --install --no-debsig --force-confnew -G ./debs/*.deb

# Restore / Upgrade the snapshot database
restore_snapshot_database

# If dpkg failed for any reason, log a failure and quit.
if [[ $? != 0 ]]; then
	cleanup_and_request_powerdown fail
fi

# Otherwise, we were sucessful!
cleanup_and_request_powerdown success

exit 0

